(*
 *                            JULIA SETS VIEWER
 *
 *
 *                     code by Vojtech Hala, 10.6.1998
 *
 *)



{$G+,N+,E+ = generate 286 code and switch on math coprocessor}

uses rat;

type
 complex=record         { real and imaginary part of complex number }
   r,i:extended;
  end;


var
 x,y,color:longint;             { the pixel }
 cbank:word;                    { current video bank }
 Granularity:word;              { granularity of graphic window in KB }
 WindowB:boolean;               { window B does/doesn't exist }
 z,c:complex;                   { complex numbers to calculate with }
 iterations:longint;            { number of iterations }
 Iter,rminS,rmaxS,iminS,imaxS,
 r,i,cr,ci:string;              { strings to be written }
 buffer:array[0..119,0..119]of byte; { buffer for rough calculation }
 rmin,rmax,imin,imax:extended;          { actual range }
 rminO,rmaxO,iminO,imaxO:extended;      { old range }
 buff:extended;                 { buffer for swaping range }
 mouse:location;                { location of a mouse }
 width:word;                    { width of zoom array (pixels) }
 gwidth,centerR,centerI:extended;{ width and the centre of visible area }

function Julia(var z:complex;c:complex):longint;
{ the Julia set fractal, input parameter c drives the form of the image }
var
 n:longint;
 r,i:real;
begin
for n:=1 to iterations do begin
  r:=z.r;i:=z.i;
  z.i:=2*r*i + c.i + n/iterations*1e-2;
  r:=r*r;i:=i*i;
  z.r:=r-i+c.r;         { z:= z^2 + c }
  if r + i > 4 then break;
 end;
Julia:=n mod iterations;
end;{Julia}

procedure PutPixel(x,y:word;color:byte);assembler;
{ in VESA mode 101H (640*480*256) }
asm
        mov  ax,y
        mov  dx,640
        mul  dx
        add  ax,x
        adc  dx,0
        mov  di,ax
        cmp  dx,cbank
        je   @BankOK
        mov  cBank,DX
        mov  ax,granularity
        mul  dl
        mov  dx,ax
        mov  ax,4f05H
        mov  bx,0
        int  10H
        cmp  WindowB,True
        jne  @BankOK
        mov  ax,4f05H
        mov  bx,1
        int  10H
@BankOK:mov  es,[SegA000]
        mov  al,color
        mov  [es:di],al
end;{PutPixel}

procedure XorPixel(x,y:word);assembler;
{ xor specified pixel with 80H }
asm
        mov  ax,y
        mov  dx,640
        mul  dx
        add  ax,x
        adc  dx,0
        mov  di,ax
        cmp  dx,cbank
        je   @BankOK
        mov  cBank,dx
        mov  ax,granularity
        mul  dl
        mov  dx,ax
        mov  ax,4f05H
        mov  bx,0
        int  10H        { switch bank in window A }
        cmp  WindowB,0
        je   @BankOK
        mov  ax,4f05H
        mov  bx,1
        int  10H        { switch bank in window B }
@BankOK:mov  es,[SegA000]
        xor  byte ptr[es:di],80H
end;{XorPixel}

procedure SetRGB(color,r,g,b:byte);assembler;
{ using 256 colors palette }
asm
 mov dx,03c8H
 mov al,color
 out dx,al
 inc dx
 mov al,r
 out dx,al
 mov al,g
 out dx,al
 mov al,b
 out dx,al
end;{SetRGB}

procedure SetTextMode;assembler;
{ returns standard text mode 80*25 }
asm
mov ax,3
int 10H
end;{SetTextMode}

procedure InitSvga;assembler;
{ Initialize VESA mode 101H }
var buf:array[0..255]of byte;
asm
mov  AX,4F02h
mov  BX,101H
int  10h         { set mode 101H }
mov cx,101H
mov ax,seg buf
mov es,ax
mov di,offset buf
mov ax,4f01H
int 10H                { get informations about vesa mode 101H }
mov al,[es:di+3]       { offset 2 - about window B }
and al,01H
mov WindowB,al         { bit 0 = window B does exist }
mov  ax,64
mov  cx,[es:di+4]      { offset 4 - window granularity in KB }
div  cl
mov  granularity,ax    { ganularity := 64 / granularity }
mov  dx,0
mov  ax,4f05H
mov  bx,0
int  10H        { set zero videobank in window A }
cmp  WindowB,true
jne  @BankOK
mov  ax,4f05H
mov  bx,1
int  10H        { set zero videobank in window B }
@BankOK:
mov  cBank,0
end;{InitSvga}

procedure PreparePalette;
{ prepare the 256 color palette to use }
var i:byte;
begin
for i:=0 to 63 do SetRGB(i,0,0,i);
for i:=64 to 127 do SetRGB(i,i-64,i-64,127-i);
for i:=128 to 191 do SetRGB(i,63,127-i,0);
for i:=0 to 63 do SetRGB(i+192,63-i,0,0);
{for i:=0 to 63 do SetRGB(i,i,0,0);
for i:=64 to 127 do SetRGB(i,63,i-64,0);
for i:=128 to 191 do SetRGB(i,63,63,i-128);
for i:=0 to 63 do SetRGB(i+192,63-i,63-i,63-i);}
end;{PreparePalette}

function ReadKey:char;assembler;
{ returns key code }
asm
mov ax,0
int 16H
or al,al
jnz @konec
mov al,ah
@konec:
end;

function KeyPressed:boolean;assembler;
{ equal to crt.KeyPressed }
asm
   mov ah,01H
   int 16H
   mov al,false
   jz  @1
   mov al,true
@1:
end;

procedure OutTextXY(x,y:byte;Txt:string);
{ writes the text onto screen, x is in [0..79], y is in [0..29]
  This works in text mode as in graphix! }
var
 i:byte;
 character:char;
begin
asm
  mov bh,0
  mov dh,y
  mov dl,x
  mov ah,02H
  int 10H { move imaginary cursor to x,y }
end;
for i:=1 to byte(Txt[0])do begin
  character:=Txt[i];
  asm
  mov al,Character
  mov bh,0              { background color }
  mov bl,128            { character color (192=white,128=yellow,64=blue...) }
  mov cx,1
  mov ah,09H
  int 10H               { write the character}
  mov bh,0
  mov dh,y
  mov dl,x
  add dl,i
  mov ah,02H
  int 10H               { move the cursor }
  end;
 end;
end;{OutTextXY}

procedure PrecalculateBuffer;
{ prepares the rough computation }
(* see documentation *)
var i,j:byte;
begin
asm
mov  dx,0
mov  cBank,dx
mov  ax,4f05H
mov  bx,0
int  10H        { set zero videobank in window A }
cmp  WindowB,true
jne  @BankOK
mov  ax,4f05H
mov  bx,1
int  10H        { set zero videobank in window B }
@BankOK:
end;
for y:=0 to 119 do
 for x:=0 to 119 do begin
   z.r:=(x-60)/120*gWidth+CenterR;{ compute the complex number }
   z.i:=(60-y)/120*gWidth+CenterI;{ in the middle of the pixel }
   buffer[x,y]:=255*Julia(z,c)div iterations;
{   for i:=0 to 3 do for j:=0 to 3 do
    PutPixel(x shl 2+135+j,y shl 2+i,buffer[x,y]);}
  end;
end;{PrecalculateBuffer}

function Stabile(x,y:byte):boolean;
{ returns true if four "rough" pixels around the specified one
  has the same value }
(* see documentation *)
var is:boolean;
  value:byte;
begin
is:=true;
value:=buffer[x,y];
if buffer[x-1,y-1]<> value then is:=false;
if buffer[x+1,y-1]<> value then is:=false;
if buffer[x-1,y+1]<> value then is:=false;
if buffer[x+1,y+1]<> value then is:=false;
stabile:=is;
end;{Stabile}

procedure SetupRange;assembler;
{ set restrictions for mouse movement using actual width of mouse cursor }
asm
mov ax,7        { set mouse movement x-range }
mov cx,144
add cx,width
shl cx,3
mov dx,624
sub dx,width
shl dx,3
int 33H
mov ax,8        { set mouse movement y-range }
mov cx,width
shl cx,3
mov dx,480
sub dx,width
shl dx,3
int 33H
end;{SetupRange}

procedure MouseCursor;
{ Draw or hide the mouse cursor using XorPixel }
var x,y:integer;
begin
asm
mov  dx,0
mov  cBank,dx
mov  ax,4f05H
mov  bx,0
int  10H        { set zero videobank in window A }
cmp  WindowB,true
jne  @BankOK
mov  ax,4f05H
mov  bx,1
int  10H        { set zero videobank in window B }
@BankOK:
end;
for x:=-width+1 to width do XorPixel(mouse.x+x,mouse.y-width);
for x:=-width+2 to width-1 do XorPixel(mouse.x+x,mouse.y+width-1);
for y:=-width+1 to width-1 do XorPixel(mouse.x-width+1,mouse.y+y);
for y:=-width+1 to width-1 do XorPixel(mouse.x+width,mouse.y+y);
for x:=-2 to 2 do XorPixel(mouse.x+x,mouse.y);
for y:=-2 to 2 do XorPixel(mouse.x,mouse.y+y);
end;{MouseCursor}

procedure ComputeTheFractal;
{ rutine to realize all the calculation }
begin
Str(rMin:15:12,rMinS);
Str(rMax:15:12,rMaxS);
Str(iMin:15:12,iMinS);
Str(iMax:15:12,iMaxS);
OutTextXY(0,0,'Range:');
OutTextXY(0,1,rMinS);
OutTextXY(0,2,iMinS+'i');
OutTextXY(0,3,rMaxS);
OutTextXY(0,4,iMaxS+'i');       { write out new range }
OutTextXY(0,29,'Preparing...');
PrecalculateBuffer;
OutTextXY(0,29,'Computing...'); { write out status }
asm
mov  dx,0
mov  cBank,dx
mov  ax,4f05H
mov  bx,0
int  10H        { set zero videobank in window A }
cmp  WindowB,true
jne  @BankOK
mov  ax,4f05H
mov  bx,1
int  10H        { set zero videobank in window B }
@BankOK:
end;
for y:=0 to 479 do begin
 if (keypressed)or(rButtonDown) then break;
 for x:=80 to 559 do begin
   z.r:=(x-320)/480*gWidth+CenterR;{ compute the complex number }
   z.i:=(240-y)/480*gWidth+CenterI;{ in the middle of the pixel }
   if not stabile((x-80)shr 2,y shr 2)
    then color:=255*Julia(z,c)div iterations
    else color:=buffer[(x-80)shr 2,y shr 2]; (* see documentation *)
   PutPixel(x+65,y,color);
  end;
end;
while keypressed do readkey;    { clear keyboard buffer }
OutTextXY(0,29,'Ready.      ');
width:=15;
SetupRange;
end;{ComputeTheFractal}

procedure GetParams;
{ get value of 'c' from command line }
var
 r,i:extended;  { two real parameters }
 err:integer;   { error code }
 ok:boolean;    { true if no problem }
begin
err:=0;ok:=true;
Val(ParamStr(1),r,err);if err<>0then ok:=false;
Val(ParamStr(2),i,err);if err<>0then ok:=false;
c.r:=0; c.i:=-0.76;
{ if command line parameter isn't valid use implicite value }
if ok then begin
  c.r:=r;
  c.i:=i;
 end;
end;{GetParams}

(*procedure Intro;
{ print file julia.txt }
var
 f:text;
 s:string;
 lines:byte;
begin
assign(f,'julia.txt');
{$I-} reset(f); {$I-}
if IoResult<>0 then exit;
{ if intro file is not accessable then continue without intro }
lines:=0;
while not eof(f) do begin
  ReadLn(f,s);
  WriteLn(s);
  inc(lines);
  if (lines=23)or eof(f) then begin
    asm
     mov     ah,1
     mov     ch,20H
     int     10H            { Hide cursor }
    end;
    writeln;
    write('Press any key to continue...');
    readkey;
    SetTextMode;        { clear screen }
    lines:=0;
   end;
 end;
Close(f);
end;{Intro}*)


{ --- The program body ---}

begin
GetParams;
(*Intro;*)
InitSvga;               { set mode 640*480*256 }
PreparePalette;         { set 256 colors RGB table }
iterations:=40;         { start with 40 iterations }
OutTextXY(0,25,'Position:');
OutTextXY(0,21,'c =');
Str(c.r:15:12,cr);
OutTextXY(0,22,cr);
Str(c.i:15:12,ci);
OutTextXY(0,23,ci+'i');
Str(iterations,iter);   { convert numbers to strings }
OutTextXY(0,28,iter+' iterations    ');
{ write out the number of iterations }
rmin:=-2;imin:=-2;rmax:=2;imax:=2;
{ basic range is the square -2-2i to 2+2i }
centerR:=(rmin+rmax)/2;centerI:=(imin+imax)/2;
gWidth:=rmax-rmin;
rminO:=0;rmaxO:=0;{ no old range }
for x:=30 to 50 do for y:=100 to 356 do PutPixel(x,436-y,y-100);
{ Draw the palette }
ComputeTheFractal;
if not InitMouse then begin
  writeln;
  writeln('Mouse driver not instaled.');
  halt(255);
 end;
SetSensitivity(8);
SetupRange;
MoveMouse(0,0);
GetPosition(mouse);
MouseCursor;
repeat
if mousemove then begin
  MouseCursor;         { hide mouse cursor }
  GetPosition(mouse);  { get new position }
  MouseCursor;         { show the cursor }
  Str((mouse.x-64-320)/480*gWidth+CenterR:17:14,r);
  Str((240-mouse.y)/480*gWidth+CenterI:17:14,i);
  OutTextXY(0,26,r);
  OutTextXY(0,27,i+'i'); { compute and write out new position }
 end;
if lButtonDown then begin
  rminO:=rmin; rmaxO:=rmax;iminO:=imin;imaxO:=imax;
  { Store old range }
  rmin:=(mouse.x-width-64-320)/480*gWidth+CenterR;
  imin:=(240-mouse.y-width)/480*gWidth+CenterI;
  rmax:=(mouse.x-64+width-320)/480*gWidth+CenterR;
  imax:=(240-mouse.y+width)/480*gWidth+CenterI;
  centerR:=(rmin+rmax)/2; centerI:=(imin+imax)/2;
  gWidth:=rmax-rmin;   { Compute new range }
  MouseCursor;
  ComputeTheFractal;
  MouseCursor;
 end;
if keypressed then case readkey of
    's':begin
        MouseCursor;
        MouseCursor;
        end;
    #27:break;                  { Esc }
    #13:begin                   { Enter }
         MouseCursor;
         ComputeTheFractal;
         MouseCursor;
        end;
    #32:begin
         rminO:=rmin; rmaxO:=rmax;iminO:=imin;imaxO:=imax;
         { Store old range }
         rmin:=(mouse.x-width-64-320)/480*gWidth+CenterR;
         imin:=(240-mouse.y-width)/480*gWidth+CenterI;
         rmax:=(mouse.x-64+width-320)/480*gWidth+CenterR;
         imax:=(240-mouse.y+width)/480*gWidth+CenterI;
         centerR:=(rmin+rmax)/2; centerI:=(imin+imax)/2;
         gWidth:=rmax-rmin;   { Compute new range }
         MouseCursor;
         ComputeTheFractal;
         MouseCursor;
        end;
    #08:if rminO<>rmaxO then begin { Backspace }
         buff:=rmin; rmin:=rminO; rminO:=buff;
         buff:=rmax; rmax:=rmaxO; rmaxO:=buff;
         buff:=imin; imin:=iminO; iminO:=buff;
         buff:=imax; imax:=imaxO; imaxO:=buff;
         centerR:=(rmin+rmax)/2; centerI:=(imin+imax)/2;
         gWidth:=rmax-rmin;   { Swap old and new range }
         MouseCursor;
         ComputeTheFractal;
         MouseCursor;
        end;
    #72:begin                   { Up arrow }
         inc(iterations);
         Str(iterations,iter);
         OutTextXY(0,28,iter+' iterations   ');
        end;
    #80:begin                   { Down arrow }
         if iterations>2 then dec(iterations);
         Str(iterations,iter);
         OutTextXY(0,28,iter+' iterations   ');
        end;
    #73:begin                   { PgUp }
         inc(iterations,10);
         Str(iterations,iter);
         OutTextXY(0,28,iter+' iterations   ');
        end;
    #81:begin                   { PgDn }
         if iterations>10 then dec(iterations,10);
         Str(iterations,iter);
         OutTextXY(0,28,iter+' iterations   ');
        end;
    #75:if width>4 then begin   { Left arrow }
         MouseCursor;
         dec(width);
         SetupRange;
         MouseCursor;
        end;
    #77:if width<240 then begin{ Right arrow }
         MouseCursor;
         inc(width);
         SetupRange;
         MouseCursor;
        end;
 end;
until rButtonDown;
SetTextMode;
end.